@@ -75,18 +75,14 @@ typedef GTypeModule XfcePanelTypeModule;
 /**
  * XFCE_PANEL_CHANNEL_NAME:
  *
- * Macro for the name of the Xfconf channel used by the panel. By default
- * this returns "xfce4-panel", but you can override this value with the
- * environment variable XFCE_PANEL_CHANNEL_NAME.
+ * Macro to return the value of xfce_panel_get_channel_name().
  *
  * See also: xfce_panel_plugin_xfconf_channel_new,
  *           xfce_panel_plugin_get_property_base
  *
  * Since: 4.8
  **/
-#define XFCE_PANEL_CHANNEL_NAME \
-  (G_LIKELY (g_getenv ("XFCE_PANEL_CHANNEL_NAME") == NULL) ? \
-      "xfce4-panel" : g_getenv ("XFCE_PANEL_CHANNEL_NAME"))
+#define XFCE_PANEL_CHANNEL_NAME (xfce_panel_get_channel_name ())
 
 
 
@@ -99,7 +95,8 @@ typedef GTypeModule XfcePanelTypeModule;
  * xfce_panel_plugin_get_property_base().
  *
  * See also: xfce_panel_plugin_get_property_base,
- *           XFCE_PANEL_PLUGIN_CHANNEL_NAME
+ *           XFCE_PANEL_PLUGIN_CHANNEL_NAME and
+ *           xfce_panel_get_channel_name
  *
  * Since: 4.8
  **/
