@@ -1426,7 +1426,7 @@ panel_application_logout (void)
   else if (g_getenv ("SESSION_MANAGER") == NULL)
     {
       if (xfce_dialog_confirm (NULL, GTK_STOCK_QUIT, NULL,
-          _("If you have started Xfce without session manager, this will close the X server."),
+          _("You have started X without session manager. Clicking Quit will close the X server."),
           _("Are you sure you want to quit the panel?")))
         command = "xfce4-panel --quit";
       else
