@@ -85,6 +85,9 @@ struct _PanelDBusService
 
   /* queue for remote-events */
   GHashTable      *remote_events;
+
+  /* whether the service is owner of the name */
+  guint            is_owner : 1;
 };
 
 typedef struct
@@ -126,21 +129,27 @@ panel_dbus_service_init (PanelDBusService *service)
 {
   GError         *error = NULL;
   DBusConnection *connection;
+  gint            result;
 
-  service->remote_events = g_hash_table_new_full (g_int_hash, g_int_equal, NULL,
-                                                  panel_dbus_service_plugin_event_free);
+  service->is_owner = FALSE;
+  service->remote_events = NULL;
 
   service->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
   if (G_LIKELY (service->connection != NULL))
     {
       /* TODO handle error */
       connection = dbus_g_connection_get_connection (service->connection);
-      dbus_bus_request_name (connection, PANEL_DBUS_NAME, 0, NULL);
+      result = dbus_bus_request_name (connection, PANEL_DBUS_NAME, DBUS_NAME_FLAG_DO_NOT_QUEUE, NULL);
+      if (result == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
+        {
+          /* yes we own the name */
+          service->is_owner = TRUE;
 
-      /* register this object */
-      dbus_g_connection_register_g_object (service->connection,
-                                           PANEL_DBUS_PATH,
-                                           G_OBJECT (service));
+          /* register this object */
+          dbus_g_connection_register_g_object (service->connection,
+                                               PANEL_DBUS_PATH,
+                                               G_OBJECT (service));
+        }
     }
   else
     {
@@ -157,8 +166,6 @@ panel_dbus_service_finalize (GObject *object)
   PanelDBusService *service = PANEL_DBUS_SERVICE (object);
   DBusConnection   *connection;
 
-  panel_return_if_fail (g_hash_table_size (service->remote_events) == 0);
-
   if (G_LIKELY (service->connection != NULL))
     {
       /* release the org.xfce.Panel name */
@@ -169,7 +176,11 @@ panel_dbus_service_finalize (GObject *object)
       dbus_g_connection_unref (service->connection);
     }
 
-  g_hash_table_destroy (service->remote_events);
+  if (service->remote_events != NULL)
+    {
+      panel_return_if_fail (g_hash_table_size (service->remote_events) == 0);
+      g_hash_table_destroy (service->remote_events);
+    }
 
   (*G_OBJECT_CLASS (panel_dbus_service_parent_class)->finalize) (object);
 }
@@ -379,6 +390,11 @@ panel_dbus_service_plugin_event (PanelDBusService  *service,
           g_value_init (&event->value, G_VALUE_TYPE (value));
           g_value_copy (value, &event->value);
 
+          /* create hash table if needed */
+          if (service->remote_events == NULL)
+            service->remote_events = g_hash_table_new_full (g_int_hash, g_int_equal, NULL,
+                                                            panel_dbus_service_plugin_event_free);
+
           g_hash_table_insert (service->remote_events, &event->handle, event);
           g_signal_connect (G_OBJECT (li->data), "remote-event-result",
               G_CALLBACK (panel_dbus_service_plugin_event_result), service);
@@ -436,6 +452,15 @@ panel_dbus_service_get (void)
 
 
 
+gboolean
+panel_dbus_service_is_owner (PanelDBusService *service)
+{
+  panel_return_val_if_fail (PANEL_IS_DBUS_SERVICE (service), FALSE);
+  return service->is_owner;
+}
+
+
+
 void
 panel_dbus_service_exit_panel (gboolean restart)
 {
