@@ -266,6 +266,7 @@ systray_manager_new (void)
 
 
 
+#if 0
 gboolean
 systray_manager_check_running (GdkScreen *screen)
 {
@@ -291,6 +292,7 @@ systray_manager_check_running (GdkScreen *screen)
   /* return result */
   return (XGetSelectionOwner (GDK_DISPLAY_XDISPLAY (display), selection_atom) != None);
 }
+#endif
 
 
 
@@ -446,13 +448,13 @@ systray_manager_unregister (SystrayManager *manager)
 
   /* remove our handling of the selection if we're the owner */
   owner = gdk_selection_owner_get_for_display (display, manager->selection_atom);
-  if (owner != invisible->window
-      || !gdk_selection_owner_set_for_display (display,
-                                               NULL,
-                                               manager->selection_atom,
-                                               gdk_x11_get_server_time (invisible->window),
-                                               TRUE))
-    panel_assert_not_reached ();
+  if (owner == invisible->window)
+    {
+      gdk_selection_owner_set_for_display (display, NULL,
+                                           manager->selection_atom,
+                                           gdk_x11_get_server_time (invisible->window),
+                                           TRUE);
+    }
 
   /* remove window filter */
   gdk_window_remove_filter (invisible->window,
